@@ -12,9 +12,10 @@ use lib "$FindBin::Bin/../lib";
 use File::Basename;
 use File::Glob;
 use File::Spec::Functions ':ALL';
-use Cwd 'realpath';
+use Cwd 'abs_path';
 use Getopt::Long;
 use IO::Compress::Gzip qw(gzip $GzipError);
+use DynaLoader;
 use Config;
 
 my $chunk_size = 32768;
@@ -23,9 +24,9 @@ my $compress = 0;
 GetOptions(
     "c|chunk-size=i"    => \$chunk_size,
     "z|compress"        => \$compress)
-    && @ARGV == 1
-        or die "Usage: $0 [-c CHUNK][-z] par > file.c\n";
-my ($par) = @ARGV;
+    && @ARGV >= 1
+        or die "Usage: $0 [-c CHUNK][-z] par fallback_embedded_file... > file.c\n";
+my ($par, @fallback_embedded_files) = @ARGV;
 
 sub is_system_lib;
 
@@ -33,22 +34,44 @@ my $dlls;
 for ($^O)
 {
     # sane platforms: use "ldd"
-    if (/linux|solaris|freebsd|openbsd/i) 
+    if (/linux|solaris|freebsd|openbsd|cygwin/i) 
     {
         print STDERR qq[# using "ldd" to find shared libraries needed by $par\n];
-        *is_system_lib = sub { shift =~ m{^(?:/usr)?/lib(?:32|64)?/} };
+        if ($^O =~ /cygwin/i)
+        {
+            chomp(my $system_root = qx( cygpath --unix '$ENV{SYSTEMROOT}' ));
+            print STDERR "### SystemRoot (as Unix path) = $system_root\n";
+            *is_system_lib = sub { shift =~ m{^/usr/bin/(?!cygcrypt\b)|^\Q$system_root\E/}i }; 
+            # NOTE: cygcrypt-0.dll is not (anymore) in the set of default Cygwin packages
+        }
+        else
+        {
+            *is_system_lib = sub { shift =~ m{^(?:/usr)?/lib(?:32|64)?/} };
+        }
 
         $dlls = ldd($par); 
+
+        # weed out system libs (but exclude the shared perl lib)
+        while  (my ($name, $path) = each %$dlls)
+        {
+            delete $dlls->{$name} if is_system_lib($path) && $name !~ /perl/;
+        }
         last;
     }
 
-    # Max OS X: use "otool -L"
-    if (/darwin/i && (qx(otool --version), $? == 0)) 
+    # Max OS X: use "otool -L" if available
+    if (/darwin/i && (qx(otool -h $par), $? == 0)) 
     {
         print STDERR qq[# using "otool -L" to find shared libraries needed by $par\n];
         *is_system_lib = sub { shift =~ m{^/usr/lib|^/System/Library/} };
 
         $dlls = otool($par); 
+
+        # weed out system libs (but exclude the shared perl lib)
+        while  (my ($name, $path) = each %$dlls)
+        {
+            delete $dlls->{$name} if is_system_lib($path) && basename($path) !~ /perl/;
+        }
         last;
     }
 
@@ -56,8 +79,8 @@ for ($^O)
     if (/mswin32/i && (qx(objdump --version), $? == 0))
     {
         print STDERR qq[# using "objdump" recusrively to find DLLs needed by $par\n];
-        my $system_root = realpath($ENV{SystemRoot});
-        *is_system_lib = sub { realpath(shift) =~ m{^\Q$system_root\E/}i };
+        my $system_root = abs_path($ENV{SystemRoot});
+        *is_system_lib = sub { abs_path(shift) =~ m{^\Q$system_root\E/}i };
 
         $dlls = objdump($par);
         last;
@@ -65,26 +88,17 @@ for ($^O)
 
     # fall back to guessing game
     print STDERR qq[# fall back to guessing what DLLs are needed by $par\n];
-    $dlls = fallback($par);
+    $dlls = { map { basename($_) => $_ } @fallback_embedded_files };
 }
 
 
-my $n = 0;
-my @embedded_files = {          # par is always the first embedded file
-    name   => basename($par),
-    size   => -s $par,
-    chunks => file2c("file$n", $par),
-};
-$n++;
+# par is always the first embedded file
+my @embedded_files;
+embed(\@embedded_files, basename($par), $par);
 
 while (my ($name, $file) = each %$dlls)
 {
-    push @embedded_files, {
-        name   => $name,
-        size   => -s $file,
-        chunks => file2c("file$n", $file),
-    };
-    $n++;
+    embed(\@embedded_files, $name, $file);
 }
 
 print "static embedded_file_t embedded_files[] = {\n";
@@ -94,6 +108,20 @@ print "  { NULL, 0, NULL }\n};";
 exit 0;
 
 
+sub embed
+{
+    my ($embedded, $name, $file) = @_;
+    print STDERR qq[# embedding "$file" as "$name"\n];
+
+    my $n = @$embedded;
+    push @$embedded, 
+    { 
+        name   => $name, 
+        size   => -s $file, 
+        chunks => file2c("file$n", $file) 
+    };
+}
+
 sub ldd
 {
     my ($file) = @_;
@@ -101,15 +129,21 @@ sub ldd
     my $out = qx(ldd $file);
     die qq["ldd $file" failed\n] unless $? == 0;
 
-    my %dlls = $out =~ /^ \s* (\S+) \s* => \s* (\S+) /gmx;
+    # NOTE: On older Linux/glibc (e.g. seen on Linux 3.2.0/glibc 2.13)
+    # ldd prints a line like
+    #    linux-vdso.so.1 =>  (0x00007fffd2ff2000)
+    # (without a pathname between "=>" and the address)
+    # while newer versions omit "=>" in this case.
+    my %dlls = $out =~ /^ \s* (\S+) \s* => \s* ( \/ \S+ ) /gmx;
 
-    # weed out system libraries (except the perl shared library)
     while (my ($name, $path) = each %dlls)
     {
-        delete $dlls{$name} unless -r $path;    # huh?
-
-        next if $name =~ /^libperl/;
-        delete $dlls{$name} if is_system_lib($path);
+        unless (-r $path)
+        {
+            warn qq[# ldd reported strange path: $path\n];
+            delete $dlls{$name};
+            next;
+        }
     }
 
     return \%dlls;
@@ -125,9 +159,7 @@ sub otool
     my $out = qx(otool -L $file);
     die qq["otool -L $file" failed\n] unless $? == 0;
 
-    return { map { basename($_) => $_ }
-                 grep { basename($_) =~ /^libperl/ || !is_system_lib($_) }
-                      $out =~ /^ \s+ (\S+) /gmx };
+    return { map { basename($_) => $_ } $out =~ /^ \s+ (\S+) /gmx };
 }
 
 sub objdump
@@ -135,7 +167,7 @@ sub objdump
     my ($path) = @_;
 
     my %dlls;;
-    _objdump($path, "", { lc realpath($path) => 1 }, \%dlls);
+    _objdump($path, "", { lc abs_path($path) => 1 }, \%dlls);
 
     # weed out system libraries
     while (my ($name, $path) = each %dlls)
@@ -157,74 +189,16 @@ sub _objdump
     {
         next if $dlls->{$dll};
 
-        my $path = _find_dll($dll) or next;
-        $dlls->{$dll} = $path;
+        my ($file) = DynaLoader::dl_findfile($dll) or next;
+        $dlls->{$dll} = $file;
 
-        next if $seen->{$path};
-        _objdump($path, "$level  ", $seen, $dlls) 
-            unless is_system_lib($path);
-        $seen->{lc $path} = 1;
+        next if $seen->{$file};
+        _objdump($file, "$level  ", $seen, $dlls) 
+            unless is_system_lib($file);
+        $seen->{lc $file} = 1;
     }
 }
 
-sub _find_dll
-{
-    my ($name) = @_;
-
-    foreach (path())
-    {
-        my $path = catfile($_, $name);
-        return realpath($path) if -r $path;
-    }
-    return;
-}
-
-# If on Windows and Perl was built with GCC 4.x, then libperl*.dll
-# may depend on some libgcc_*.dll (e.g. Strawberry Perl 5.12).
-# This libgcc_*.dll has to be included into with any packed executable 
-# in the same way as libperl*.dll itself, otherwise a packed executable
-# won't run when libgcc_*.dll isn't installed.
-# The same holds for libstdc++*.dll (e.g. Strawberry Perl 5.16).
-
-sub fallback
-{
-    my ($file) = @_;
-
-    my @libs;
-    if ($^O eq 'MSWin32'
-        and defined $Config{gccversion}             # gcc version >= 4.x was used
-        and $Config{gccversion} =~ m{\A(\d+)}ms && $1 >= 4) 
-    {
-        push @libs, _find_dll_glob("libgcc_*.$Config{so}"),
-                    _find_dll_glob("libwinpthread*.$Config{so}");
-    }
-
-    my $ld = $Config{ld} || (($^O eq 'MSWin32') ? 'link.exe' : $Config{cc});
-    $ld = $Config{cc} if ($^O =~ /^(?:dec_osf|aix|hpux)$/);
-    if ($ld =~ /(\b|-)g\+\+(-.*)?(\.exe)?$/)        # g++ was used to link
-    {
-        push @libs, _find_dll_glob("libstdc++*.$Config{so}");
-    }
-
-    return { map { basename($_) => $_ } grep { defined } @libs };
-}
-
-sub _find_dll_glob
-{
-    my ($dll_glob) = @_;
-
-    # look for $dll_glob
-    # - in the same directory as the perl executable itself
-    # - in the same directory as gcc (only useful if it's an absolute path)
-    # - in PATH
-    my ($dll_path) = map { File::Glob::bsd_glob(catfile($_, $dll_glob)) }
-                         dirname($^X),
-                         dirname($Config{cc}),
-                         path();
-    return $dll_path;
-}
-
-
 sub file2c
 {
     my ($prefix, $path) = @_;
